﻿using System.Globalization;

using TestLibrary.Common;

namespace TestLibrary.Validator;

/// <summary>
/// 라이센스 키 검증자
/// </summary>
public class LicenseKeyValidator : LincenseKeyBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 키 검증하기 - ValidateKey(key, keyByteSetArray, totalKeyByteSetCount, keyListBlackListed)

    /// <summary>
    /// 키 검증하기
    /// </summary>
    /// <param name="key">키</param>
    /// <param name="keyByteSetArray">키 바이트 세트 배열</param>
    /// <param name="totalKeyByteSetCount">전체 키 바이트 세트 수</param>
    /// <param name="keyListBlackListed">블랙 리스트 키 리스트</param>
    /// <returns>라이센스 키 검증 결과</returns>
    public LicenseKeyValidationResult ValidateKey(string key, KeyByteSet[] keyByteSetArray, int totalKeyByteSetCount, string[] keyListBlackListed)
    {
        key = NormalizeKey(key);

        LicenseKeyValidationResult validationResult = LicenseKeyValidationResult.KeyIsInvalid;

        bool passChecksum = ValidateKeyChecksum(key, totalKeyByteSetCount);

        if(passChecksum)
        {
            if(keyListBlackListed != null && keyListBlackListed.Length > 0)
            {
                for(int i = 0; i < keyListBlackListed.Length; i++)
                {
                    if(key.StartsWith(keyListBlackListed[i]))
                    {
                        validationResult = LicenseKeyValidationResult.KeyBlackListed;
                    }
                }
            }

            if(validationResult != LicenseKeyValidationResult.KeyBlackListed)
            {
                validationResult = LicenseKeyValidationResult.KeyPhoney;

                int seed;

                bool seedParsed = int.TryParse(key.Substring(0, 8), NumberStyles.HexNumber, null, out seed);

                if(seedParsed)
                {
                    foreach(KeyByteSet keyByteSet in keyByteSetArray)
                    {
                        int pattern = GetPattern(keyByteSet.KeyByteNumber);

                        if(pattern - 1 > key.Length)
                        {
                            throw new InvalidOperationException
                            (
                                "The KeyByte check position is out of range. " +
                                "You may have specified a check KeyByteNumber that did not exist in the original key generation."
                            );
                        }

                        string pattenValue = key.Substring(pattern, 2);

                        byte keyByte = GetKeyByte(seed, keyByteSet.KeyByteA, keyByteSet.KeyByteB, keyByteSet.KeyByteC);

                        if(pattenValue != keyByte.ToString("X2"))
                        {
                            return validationResult;
                        }
                    }

                    validationResult = LicenseKeyValidationResult.KeyIsValid;
                }
            }
        }

        return validationResult;
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 패턴 구하기 - GetPattern(keyByteNumber)

    /// <summary>
    /// 패턴 구하기
    /// </summary>
    /// <param name="keyByteNumber">키 바이트 번호</param>
    /// <returns>패턴</returns>
    private int GetPattern(int keyByteNumber)
    {
        return (keyByteNumber * 2) + 6;
    }

    #endregion
    #region 키 체크썸 검증하기 - ValidateKeyChecksum(key, totalKeyByteSetCount)

    /// <summary>
    /// 키 체크썸 검증하기
    /// </summary>
    /// <param name="key">키</param>
    /// <param name="totalKeyByteSetCount">전체 키 바이트 세트 수</param>
    /// <returns>키 체크썸 검증 결과</returns>
    private bool ValidateKeyChecksum(string key, int totalKeyByteSetCount)
    {
        bool result = false;

        string keyNormalized = NormalizeKey(key);

        if(keyNormalized.Length == (8 + 4 + (2 * totalKeyByteSetCount)))
        {
            int keyLessChecksumLength = keyNormalized.Length - 4;

            string checksum = keyNormalized.Substring(keyLessChecksumLength, 4);

            string keyWithoutChecksum = keyNormalized.Substring(0, keyLessChecksumLength);

            result = GetChecksum(keyWithoutChecksum) == checksum;
        }

        return result;
    }

    #endregion
}